import logging


def file_not_found_error(filename, msg=""):
    """抛出找不到文件异常"""
    logging.error(f"找不到文件：{filename}\n{msg}")
    exit(-1)


def dir_not_found_error(path, msg=""):
    """抛出找不到目录异常"""
    logging.error(f"找不到目录：{path}\n{msg}")
    exit(-1)


def error(msg):
    """抛出通用异常信息"""
    logging.error(msg)
    exit(-1)



import logging
import os
import time

import mistune

from ..constants.layouts_html import book_summary_3_head, book_summary_3_chapter_active, \
    book_summary_3_chaptere, book_summary_3_sub_chapter_active, book_summary_3_sub_chaptere
from ..models.book import Book
from ..utils.path import get_pure_path, get_filename_not_ext


class SummaryRenderer(mistune.Renderer):
    def __init__(self):
        super().__init__()
        self.toc_tree = []

    def link(self, link, title, text):
        """Rendering a given link with content and title.

        :param link: href link for ``<a>`` tag.
        :param title: title content for `title` attribute.
        :param text: text content for description.
        """
        link = mistune.escape_link(link)
        return f'["{link}","{title}","{text}"]'

    def paragraph(self, text):
        """Rendering paragraph tags. Like ``<p>``."""
        return text.strip(' ')

    def header(self, text, level, raw=None):
        """Rendering header/heading tags like ``<h1>`` ``<h2>``.

        :param text: rendered text content for the header.
        :param level: a number for the header level, for example: 1.
        :param raw: raw text content of the header.
        """
        self.toc_tree.append([text, []])
        return f'{{"header":["{text}"]}}'

    def list(self, body, ordered=True):
        """Rendering list tags like ``<ul>`` and ``<ol>``.

        :param body: body contents of the list.
        :param ordered: whether this list is ordered or not.
        """
        body = f'[{body}]'
        if len(self.toc_tree) == 0:
            self.toc_tree.append(["", []])

        self.toc_tree[-1][1] = body
        return f"{body}"

    def list_item(self, text):
        """Rendering list item snippet. Like ``<li>``."""
        return text.replace("\\", r"/")

    def newline(self):
        """Rendering newline element."""
        return ''


def renderer_summary(book: Book):
    """生成目录结构

    对每个页面独立生成目录"""
    P_list = []
    start = time.time()

    current_count = 0
    for current_level, current_ref, current_title in book.summary_level_list:
        ret = book.pool.submit(_read_summary, book.book_output, book.summary,
                               current_level, current_ref, current_count, book.summary_level_list)
        logging.debug(f"对每个页面独立生成目录：{current_level, current_title, current_ref}")
        current_count += 1
        P_list.append(ret)

    for ret in P_list:
        book.summary_classify_list.update(ret.result())

    end = time.time()
    logging.info(f"生成 {current_count} 个目录结构，耗时：{end - start}s")
    pass


def _read_summary(book_output, book_summary, current_level, current_ref, current_count, summary_level_list):
    summary = ""
    summary_classify_list = {current_level: {
        "title": "",
        "level": current_level,
        "prev_title": "",
        "prev_relative_path": "",
        "next_title": "",
        "next_relative_path": "",
        "summary": summary
    }}
    for item in book_summary:
        title = item.get("title", "")
        articles = item.get("articles", "")
        # data_level = item.get("data_level")
        summary += book_summary_3_head.substitute(title=title)
        if len(articles) == 0:
            continue

        summary += _iter_summary(book_output, summary_classify_list, articles, current_level, current_ref,
                                 current_count, summary_level_list)
    summary_classify_list[current_level]["summary"] = summary
    return summary_classify_list


def _iter_summary(book_output, summary_classify_list, old_articles,
                  current_level, current_ref, current_count, summary_level_list: list):
    summary_sub = ""
    for item in old_articles:
        title = item.get("title", "")
        ref = item.get("ref", "")
        articles = item.get("articles", "")
        data_level = item.get("data_level", "")

        tmp_dict = {
            'title': title,
            'data_level': data_level,
            'data_path': get_relpath(book_output, ref, current_ref),
        }
        if len(articles) == 0:
            summary_sub += data_level == current_level \
                           and book_summary_3_chapter_active.substitute(tmp_dict) \
                           or book_summary_3_chaptere.substitute(tmp_dict)
        else:
            tmp_dict['chapter'] = _iter_summary(book_output, summary_classify_list,
                                                articles, current_level, current_ref, current_count, summary_level_list)
            summary_sub += data_level == current_level \
                           and book_summary_3_sub_chapter_active.substitute(tmp_dict) \
                           or book_summary_3_sub_chaptere.substitute(tmp_dict)

        if data_level == current_level:
            summary_classify_list[current_level]["title"] = title
            summary_classify_list[current_level]["href"] = ref
            summary_classify_list[current_level]["basePath"] = get_relative_path(book_output, ref)
            if current_count > 0:
                summary_classify_list[current_level]["prev_title"] = summary_level_list[current_count - 1][2]
                summary_classify_list[current_level]["prev_relative_path"] = get_relpath(
                    book_output, summary_level_list[current_count - 1][1], current_ref
                ) + ".html"
            if current_count < len(summary_level_list) - 1:
                summary_classify_list[current_level]["next_title"] = summary_level_list[current_count + 1][2]
                summary_classify_list[current_level]["next_relative_path"] = get_relpath(
                    book_output, summary_level_list[current_count + 1][1], current_ref
                ) + ".html"
    return summary_sub


def get_relpath(book_output, ref: str, current_ref: str):
    """转换为相对路径"""
    _ref = ref
    if os.path.basename(_ref).lower() == "readme.md":
        if os.path.dirname(ref):
            _ref = os.path.join(os.path.relpath(os.path.dirname(ref)), "index.md")
        else:
            _ref = "index.md"
    return get_pure_path(get_filename_not_ext(
        os.path.relpath(
            os.path.join(book_output, _ref),
            os.path.dirname(os.path.join(book_output, current_ref))
        )
    ))


def get_relative_path(book_output, ref: str):
    """转换本页面相对于根的相对路径"""
    return get_pure_path(os.path.relpath(book_output, os.path.dirname(os.path.join(book_output, ref))))



import argparse
import sys


def cmd_argument():
    """处理控制台输入参数

    :return:
    """
    if len(sys.argv) == 1:
        sys.argv.append("--help")

    parser = argparse.ArgumentParser(description='编译 Markdown 书籍为 Html')

    parser.add_argument('-b', dest='build', action='store_true', default=False, help='编译书籍')
    parser.add_argument('--log', dest='log',
                        choices={"debug", "info", "warn", "error", "disabled"}, default="info",
                        help="最小日志级别显示（默认值为 info; 可选值：debug, info, warn, error, disabled）")
    parser.add_argument(dest='book', metavar='book', nargs="?",
                        default=".", help="书籍路径，默认当前目录")
    parser.add_argument(dest='output', metavar='output',
                        nargs="?", default="_book",
                        help="输出路径默认为书籍路径下 _book")
    parser.add_argument('-ba', '--base_assets', dest="base_assets", default=None,
                        help="本站资源附加基础目录，内部测试用，误用！")
    parser.add_argument('--assets', dest="assets", default=None,
                        help="释放资源文件到指定目录")

    args = parser.parse_args()
    return args



import os
import re

from ..parse.parse_markdown.file_imports import process_file_import
from ..parse.parse_markdown.lexers import Block_Lexer, CustomMarkdown, Inline_Lexer
from ..renderer.renderer_file import FileRenderer


def parse_file(file, base_path):
    """解析文件"""
    with open(file, encoding="utf-8") as f:
        page = f.read()
    dirname = os.path.dirname(file)

    # 处理引入文件
    page, assets_img = process_file_import(dirname, page, base_path)

    renderer = FileRenderer()
    inline = Inline_Lexer(renderer)
    block = Block_Lexer()
    markdown = CustomMarkdown(
        renderer=renderer,
        inline=inline,
        block=block
    )

    page = markdown(page)

    return (page, renderer.toc_tree, renderer.tag_katex, renderer.tag_mermaid,
            renderer.tag_prism, renderer.tag_lightbox, assets_img)


def sectionx(page, count=0):
    """隐藏答案框

    :param page:
    :return:
    """
    r = re.search(r"<!--sec([\s\S]+?)ces-->([\s\S]+?)<!--endsec-->", page, re.S)
    if r:
        title = ""
        show = False
        for i in r.group(1).split():
            if "title" in i:
                title = i.split("=")[1].strip("'").strip('"')
            elif "show" in i:
                show = i.split("=")[1].lower() == 'true'

        html = f'<sec data-title="{title}"><div class="panel panel-default"><div class="panel-heading"><b>{title}' \
            f'<a class="pull-right section atTitle btn btn-default {"sec-show" if show else ""}" target="sectionx{count}">' \
            f'<span class="fa {"fa-angle-up" if show else "fa-angle-down"}" /></a></b></div>' \
            f'<div class="panel-collapse {"in" if show else "collapse"}" id="sectionx{count}">' \
            f'<div class="panel-body">{r.group(2)}</div></div></div></sec>'

        page = page.replace(r.group(0), html)
        return sectionx(page, count + 1)
    else:
        return page



lang_dict = {
    "1": [
        "groff"
    ],
    "2": [
        "groff"
    ],
    "3": [
        "groff"
    ],
    "4": [
        "groff"
    ],
    "5": [
        "groff"
    ],
    "6": [
        "groff"
    ],
    "7": [
        "groff"
    ],
    "abap": [
        "abap"
    ],
    "asc": [
        "ags script",
        "asciidoc",
        "public key"
    ],
    "ash": [
        "ags script"
    ],
    "ampl": [
        "ampl"
    ],
    "g4": [
        "antlr"
    ],
    "apl": [
        "apl"
    ],
    "dyalog": [
        "apl"
    ],
    "asp": [
        "asp"
    ],
    "asax": [
        "asp"
    ],
    "ascx": [
        "asp"
    ],
    "ashx": [
        "asp"
    ],
    "asmx": [
        "asp"
    ],
    "aspx": [
        "asp"
    ],
    "axd": [
        "asp"
    ],
    "dats": [
        "ats"
    ],
    "hats": [
        "ats"
    ],
    "sats": [
        "ats"
    ],
    "as": [
        "actionscript"
    ],
    "adb": [
        "ada"
    ],
    "ada": [
        "ada"
    ],
    "ads": [
        "ada"
    ],
    "agda": [
        "agda"
    ],
    "als": [
        "alloy"
    ],
    "apacheconf": [
        "apacheconf"
    ],
    "cls": [
        "apex",
        "openedge abl",
        "tex",
        "visual basic"
    ],
    "applescript": [
        "applescript"
    ],
    "scpt": [
        "applescript"
    ],
    "arc": [
        "arc"
    ],
    "ino": [
        "arduino"
    ],
    "asciidoc": [
        "asciidoc"
    ],
    "adoc": [
        "asciidoc"
    ],
    "aj": [
        "aspectj"
    ],
    "asm": [
        "assembly"
    ],
    "a51": [
        "assembly"
    ],
    "nasm": [
        "assembly"
    ],
    "aug": [
        "augeas"
    ],
    "ahk": [
        "autohotkey"
    ],
    "ahkl": [
        "autohotkey"
    ],
    "au3": [
        "autoit"
    ],
    "awk": [
        "awk"
    ],
    "auk": [
        "awk"
    ],
    "gawk": [
        "awk"
    ],
    "mawk": [
        "awk"
    ],
    "nawk": [
        "awk"
    ],
    "bat": [
        "batchfile"
    ],
    "cmd": [
        "batchfile"
    ],
    "befunge": [
        "befunge"
    ],
    "y": [
        "bison"
    ],
    "bb": [
        "bitbake",
        "blitzbasic"
    ],
    "decls": [
        "blitzbasic"
    ],
    "bmx": [
        "blitzmax"
    ],
    "bsv": [
        "bluespec"
    ],
    "boo": [
        "boo"
    ],
    "b": [
        "brainfuck",
        "limbo"
    ],
    "bf": [
        "brainfuck"
    ],
    "brs": [
        "brightscript"
    ],
    "bro": [
        "bro"
    ],
    "c": [
        "c"
    ],
    "cats": [
        "c"
    ],
    "h": [
        "c",
        "c++",
        "objective-c"
    ],
    "idc": [
        "c"
    ],
    "w": [
        "c"
    ],
    "cs": [
        "c#",
        "smalltalk"
    ],
    "cshtml": [
        "c#"
    ],
    "csx": [
        "c#"
    ],
    "cpp": [
        "c++"
    ],
    "c++": [
        "c++"
    ],
    "cc": [
        "c++"
    ],
    "cp": [
        "c++",
        "component pascal"
    ],
    "cxx": [
        "c++"
    ],
    "h++": [
        "c++"
    ],
    "hh": [
        "c++",
        "hack"
    ],
    "hpp": [
        "c++"
    ],
    "hxx": [
        "c++"
    ],
    "inl": [
        "c++"
    ],
    "ipp": [
        "c++"
    ],
    "tcc": [
        "c++"
    ],
    "tpp": [
        "c++"
    ],
    "c-objdump": [
        "c-objdump"
    ],
    "chs": [
        "c2hs haskell"
    ],
    "clp": [
        "clips"
    ],
    "cmake": [
        "cmake"
    ],
    "cmake.in": [
        "cmake"
    ],
    "cob": [
        "cobol"
    ],
    "cbl": [
        "cobol"
    ],
    "ccp": [
        "cobol"
    ],
    "cobol": [
        "cobol"
    ],
    "cpy": [
        "cobol"
    ],
    "css": [
        "css"
    ],
    "capnp": [
        "cap'n proto"
    ],
    "mss": [
        "cartocss"
    ],
    "ceylon": [
        "ceylon"
    ],
    "chpl": [
        "chapel"
    ],
    "ck": [
        "chuck"
    ],
    "cirru": [
        "cirru"
    ],
    "clw": [
        "clarion"
    ],
    "icl": [
        "clean"
    ],
    "dcl": [
        "clean"
    ],
    "clj": [
        "clojure"
    ],
    "boot": [
        "clojure"
    ],
    "cl2": [
        "clojure"
    ],
    "cljc": [
        "clojure"
    ],
    "cljs": [
        "clojure"
    ],
    "cljs.hl": [
        "clojure"
    ],
    "cljscm": [
        "clojure"
    ],
    "cljx": [
        "clojure"
    ],
    "hic": [
        "clojure"
    ],
    "coffee": [
        "coffeescript"
    ],
    "_coffee": [
        "coffeescript"
    ],
    "cjsx": [
        "coffeescript"
    ],
    "cson": [
        "coffeescript"
    ],
    "iced": [
        "coffeescript"
    ],
    "cfm": [
        "coldfusion"
    ],
    "cfml": [
        "coldfusion"
    ],
    "cfc": [
        "coldfusion cfc"
    ],
    "lisp": [
        "common lisp",
        "newlisp"
    ],
    "asd": [
        "common lisp"
    ],
    "cl": [
        "common lisp",
        "cool",
        "opencl"
    ],
    "lsp": [
        "common lisp",
        "newlisp"
    ],
    "ny": [
        "common lisp"
    ],
    "podsl": [
        "common lisp"
    ],
    "cps": [
        "component pascal"
    ],
    "coq": [
        "coq"
    ],
    "v": [
        "coq",
        "verilog"
    ],
    "cppobjdump": [
        "cpp-objdump"
    ],
    "c++-objdump": [
        "cpp-objdump"
    ],
    "c++objdump": [
        "cpp-objdump"
    ],
    "cpp-objdump": [
        "cpp-objdump"
    ],
    "cxx-objdump": [
        "cpp-objdump"
    ],
    "creole": [
        "creole"
    ],
    "cr": [
        "crystal"
    ],
    "feature": [
        "cucumber"
    ],
    "cu": [
        "cuda"
    ],
    "cuh": [
        "cuda"
    ],
    "cy": [
        "cycript"
    ],
    "pyx": [
        "cython"
    ],
    "pxd": [
        "cython"
    ],
    "pxi": [
        "cython"
    ],
    "d": [
        "d",
        "dtrace",
        "makefile"
    ],
    "di": [
        "d"
    ],
    "d-objdump": [
        "d-objdump"
    ],
    "com": [
        "digital command language"
    ],
    "dm": [
        "dm"
    ],
    "darcspatch": [
        "darcs patch"
    ],
    "dpatch": [
        "darcs patch"
    ],
    "dart": [
        "dart"
    ],
    "diff": [
        "diff"
    ],
    "patch": [
        "diff"
    ],
    "dockerfile": [
        "dockerfile"
    ],
    "djs": [
        "dogescript"
    ],
    "dylan": [
        "dylan"
    ],
    "dyl": [
        "dylan"
    ],
    "intr": [
        "dylan"
    ],
    "lid": [
        "dylan"
    ],
    "E": [
        "e"
    ],
    "ecl": [
        "ecl",
        "prolog"
    ],
    "eclxml": [
        "ecl"
    ],
    "sch": [
        "eagle",
        "kicad"
    ],
    "brd": [
        "eagle"
    ],
    "epj": [
        "ecere projects"
    ],
    "e": [
        "eiffel"
    ],
    "ex": [
        "elixir"
    ],
    "exs": [
        "elixir"
    ],
    "elm": [
        "elm"
    ],
    "el": [
        "emacs lisp"
    ],
    "emacs": [
        "emacs lisp"
    ],
    "emacs.desktop": [
        "emacs lisp"
    ],
    "em": [
        "emberscript"
    ],
    "emberscript": [
        "emberscript"
    ],
    "erl": [
        "erlang"
    ],
    "es": [
        "erlang"
    ],
    "escript": [
        "erlang"
    ],
    "hrl": [
        "erlang"
    ],
    "fs": [
        "f#",
        "filterscript",
        "forth",
        "glsl"
    ],
    "fsi": [
        "f#"
    ],
    "fsx": [
        "f#"
    ],
    "fx": [
        "flux"
    ],
    "flux": [
        "flux"
    ],
    "f90": [
        "fortran"
    ],
    "f": [
        "fortran",
        "forth"
    ],
    "f03": [
        "fortran"
    ],
    "f08": [
        "fortran"
    ],
    "f77": [
        "fortran"
    ],
    "f95": [
        "fortran"
    ],
    "for": [
        "fortran",
        "formatted",
        "forth"
    ],
    "fpp": [
        "fortran"
    ],
    "factor": [
        "factor"
    ],
    "fy": [
        "fancy"
    ],
    "fancypack": [
        "fancy"
    ],
    "fan": [
        "fantom"
    ],
    "fth": [
        "forth"
    ],
    "4th": [
        "forth"
    ],
    "forth": [
        "forth"
    ],
    "fr": [
        "forth",
        "frege",
        "text"
    ],
    "frt": [
        "forth"
    ],
    "g": [
        "g-code",
        "gap"
    ],
    "gco": [
        "g-code"
    ],
    "gcode": [
        "g-code"
    ],
    "gms": [
        "gams"
    ],
    "gap": [
        "gap"
    ],
    "gd": [
        "gap",
        "gdscript"
    ],
    "gi": [
        "gap"
    ],
    "tst": [
        "gap",
        "scilab"
    ],
    "s": [
        "gas"
    ],
    "glsl": [
        "glsl"
    ],
    "fp": [
        "glsl"
    ],
    "frag": [
        "glsl",
        "javascript"
    ],
    "frg": [
        "glsl"
    ],
    "fshader": [
        "glsl"
    ],
    "geo": [
        "glsl"
    ],
    "geom": [
        "glsl"
    ],
    "glslv": [
        "glsl"
    ],
    "gshader": [
        "glsl"
    ],
    "shader": [
        "glsl"
    ],
    "vert": [
        "glsl"
    ],
    "vrx": [
        "glsl"
    ],
    "vshader": [
        "glsl"
    ],
    "gml": [
        "game maker language",
        "graph modeling language"
    ],
    "kid": [
        "genshi"
    ],
    "ebuild": [
        "gentoo ebuild"
    ],
    "eclass": [
        "gentoo eclass"
    ],
    "po": [
        "gettext catalog"
    ],
    "pot": [
        "gettext catalog"
    ],
    "glf": [
        "glyph"
    ],
    "gp": [
        "gnuplot"
    ],
    "gnu": [
        "gnuplot"
    ],
    "gnuplot": [
        "gnuplot"
    ],
    "plot": [
        "gnuplot"
    ],
    "plt": [
        "gnuplot"
    ],
    "go": [
        "go"
    ],
    "golo": [
        "golo"
    ],
    "gs": [
        "gosu",
        "javascript"
    ],
    "gst": [
        "gosu"
    ],
    "gsx": [
        "gosu"
    ],
    "vark": [
        "gosu"
    ],
    "grace": [
        "grace"
    ],
    "gradle": [
        "gradle"
    ],
    "gf": [
        "grammatical framework"
    ],
    "dot": [
        "graphviz (dot)"
    ],
    "gv": [
        "graphviz (dot)"
    ],
    "man": [
        "groff"
    ],
    "groovy": [
        "groovy"
    ],
    "grt": [
        "groovy"
    ],
    "gtpl": [
        "groovy"
    ],
    "gvy": [
        "groovy"
    ],
    "gsp": [
        "groovy server pages"
    ],
    "html": [
        "html"
    ],
    "htm": [
        "html"
    ],
    "html.hl": [
        "html"
    ],
    "st": [
        "html",
        "smalltalk"
    ],
    "xht": [
        "html"
    ],
    "xhtml": [
        "html"
    ],
    "mustache": [
        "html+django"
    ],
    "jinja": [
        "html+django"
    ],
    "erb": [
        "html+erb"
    ],
    "erb.deface": [
        "html+erb"
    ],
    "phtml": [
        "html+php"
    ],
    "http": [
        "http"
    ],
    "php": [
        "hack",
        "php"
    ],
    "haml": [
        "haml"
    ],
    "haml.deface": [
        "haml"
    ],
    "handlebars": [
        "handlebars"
    ],
    "hbs": [
        "handlebars"
    ],
    "hb": [
        "harbour"
    ],
    "hs": [
        "haskell"
    ],
    "hsc": [
        "haskell"
    ],
    "hx": [
        "haxe"
    ],
    "hxsl": [
        "haxe"
    ],
    "hy": [
        "hy"
    ],
    "pro": [
        "idl",
        "ini",
        "prolog",
        "qmake"
    ],
    "dlm": [
        "idl"
    ],
    "ipf": [
        "igor pro"
    ],
    "ini": [
        "ini"
    ],
    "cfg": [
        "ini"
    ],
    "prefs": [
        "ini"
    ],
    "properties": [
        "ini"
    ],
    "irclog": [
        "irc log"
    ],
    "weechatlog": [
        "irc log"
    ],
    "idr": [
        "idris"
    ],
    "lidr": [
        "idris"
    ],
    "ni": [
        "inform 7"
    ],
    "i7x": [
        "inform 7"
    ],
    "iss": [
        "inno setup"
    ],
    "io": [
        "io"
    ],
    "ik": [
        "ioke"
    ],
    "thy": [
        "isabelle"
    ],
    "ijs": [
        "j"
    ],
    "flex": [
        "jflex"
    ],
    "jflex": [
        "jflex"
    ],
    "json": [
        "json"
    ],
    "lock": [
        "json"
    ],
    "json5": [
        "json5"
    ],
    "jsonld": [
        "jsonld"
    ],
    "jq": [
        "jsoniq"
    ],
    "jade": [
        "jade"
    ],
    "j": [
        "jasmin",
        "objective-j"
    ],
    "java": [
        "java"
    ],
    "jsp": [
        "java server pages"
    ],
    "js": [
        "javascript"
    ],
    "_js": [
        "javascript"
    ],
    "bones": [
        "javascript"
    ],
    "es6": [
        "javascript"
    ],
    "jake": [
        "javascript"
    ],
    "jsb": [
        "javascript"
    ],
    "jsfl": [
        "javascript"
    ],
    "jsm": [
        "javascript"
    ],
    "jss": [
        "javascript"
    ],
    "jsx": [
        "javascript"
    ],
    "njs": [
        "javascript"
    ],
    "pac": [
        "javascript"
    ],
    "sjs": [
        "javascript"
    ],
    "ssjs": [
        "javascript"
    ],
    "sublime-build": [
        "javascript"
    ],
    "sublime-commands": [
        "javascript"
    ],
    "sublime-completions": [
        "javascript"
    ],
    "sublime-keymap": [
        "javascript"
    ],
    "sublime-macro": [
        "javascript"
    ],
    "sublime-menu": [
        "javascript"
    ],
    "sublime-mousemap": [
        "javascript"
    ],
    "sublime-project": [
        "javascript"
    ],
    "sublime-settings": [
        "javascript"
    ],
    "sublime-theme": [
        "javascript"
    ],
    "sublime-workspace": [
        "javascript"
    ],
    "sublime_metrics": [
        "javascript"
    ],
    "sublime_session": [
        "javascript"
    ],
    "xsjs": [
        "javascript"
    ],
    "xsjslib": [
        "javascript"
    ],
    "jl": [
        "julia"
    ],
    "krl": [
        "krl"
    ],
    "kit": [
        "kit"
    ],
    "kt": [
        "kotlin"
    ],
    "ktm": [
        "kotlin"
    ],
    "kts": [
        "kotlin"
    ],
    "lfe": [
        "lfe"
    ],
    "ll": [
        "llvm"
    ],
    "lol": [
        "lolcode"
    ],
    "lsl": [
        "lsl"
    ],
    "lvproj": [
        "labview"
    ],
    "lasso": [
        "lasso"
    ],
    "las": [
        "lasso"
    ],
    "lasso8": [
        "lasso"
    ],
    "lasso9": [
        "lasso"
    ],
    "ldml": [
        "lasso"
    ],
    "latte": [
        "latte"
    ],
    "lean": [
        "lean"
    ],
    "hlean": [
        "lean"
    ],
    "less": [
        "less"
    ],
    "ly": [
        "lilypond"
    ],
    "ily": [
        "lilypond"
    ],
    "m": [
        "limbo",
        "m",
        "muf",
        "mathematica",
        "matlab",
        "mercury",
        "objective-c"
    ],
    "ld": [
        "linker script"
    ],
    "lds": [
        "linker script"
    ],
    "liquid": [
        "liquid"
    ],
    "lagda": [
        "literate agda"
    ],
    "litcoffee": [
        "literate coffeescript"
    ],
    "lhs": [
        "literate haskell"
    ],
    "ls": [
        "livescript",
        "loomscript"
    ],
    "_ls": [
        "livescript"
    ],
    "xm": [
        "logos"
    ],
    "x": [
        "logos"
    ],
    "xi": [
        "logos"
    ],
    "lgt": [
        "logtalk"
    ],
    "logtalk": [
        "logtalk"
    ],
    "lookml": [
        "lookml"
    ],
    "lua": [
        "lua"
    ],
    "fcgi": [
        "lua",
        "php",
        "perl",
        "python",
        "ruby",
        "shell"
    ],
    "nse": [
        "lua"
    ],
    "pd_lua": [
        "lua"
    ],
    "rbxs": [
        "lua"
    ],
    "wlua": [
        "lua"
    ],
    "mumps": [
        "m"
    ],
    "mtml": [
        "mtml"
    ],
    "muf": [
        "muf"
    ],
    "mak": [
        "makefile"
    ],
    "mk": [
        "makefile"
    ],
    "mako": [
        "mako"
    ],
    "mao": [
        "mako"
    ],
    "md": [
        "markdown"
    ],
    "markdown": [
        "markdown"
    ],
    "mkd": [
        "markdown"
    ],
    "mkdn": [
        "markdown"
    ],
    "mkdown": [
        "markdown"
    ],
    "ron": [
        "markdown"
    ],
    "mask": [
        "mask"
    ],
    "mathematica": [
        "mathematica"
    ],
    "cdf": [
        "mathematica"
    ],
    "ma": [
        "mathematica"
    ],
    "nb": [
        "mathematica"
    ],
    "nbp": [
        "mathematica"
    ],
    "wl": [
        "mathematica"
    ],
    "wlt": [
        "mathematica"
    ],
    "matlab": [
        "matlab"
    ],
    "maxpat": [
        "max"
    ],
    "maxhelp": [
        "max"
    ],
    "maxproj": [
        "max"
    ],
    "mxt": [
        "max"
    ],
    "pat": [
        "max"
    ],
    "mediawiki": [
        "mediawiki"
    ],
    "moo": [
        "mercury",
        "moocode"
    ],
    "minid": [
        "minid"
    ],
    "druby": [
        "mirah"
    ],
    "duby": [
        "mirah"
    ],
    "mir": [
        "mirah"
    ],
    "mirah": [
        "mirah"
    ],
    "mo": [
        "modelica"
    ],
    "mms": [
        "module management system"
    ],
    "mmk": [
        "module management system"
    ],
    "monkey": [
        "monkey"
    ],
    "moon": [
        "moonscript"
    ],
    "myt": [
        "myghty"
    ],
    "nl": [
        "nl",
        "newlisp"
    ],
    "nsi": [
        "nsis"
    ],
    "nsh": [
        "nsis"
    ],
    "n": [
        "nemerle"
    ],
    "axs": [
        "netlinx"
    ],
    "axi": [
        "netlinx"
    ],
    "axs.erb": [
        "netlinx+erb"
    ],
    "axi.erb": [
        "netlinx+erb"
    ],
    "nlogo": [
        "netlogo"
    ],
    "nginxconf": [
        "nginx"
    ],
    "nim": [
        "nimrod"
    ],
    "nimrod": [
        "nimrod"
    ],
    "ninja": [
        "ninja"
    ],
    "nit": [
        "nit"
    ],
    "nix": [
        "nix"
    ],
    "nu": [
        "nu"
    ],
    "numpy": [
        "numpy"
    ],
    "numpyw": [
        "numpy"
    ],
    "numsc": [
        "numpy"
    ],
    "ml": [
        "ocaml"
    ],
    "eliom": [
        "ocaml"
    ],
    "eliomi": [
        "ocaml"
    ],
    "ml4": [
        "ocaml"
    ],
    "mli": [
        "ocaml"
    ],
    "mll": [
        "ocaml"
    ],
    "mly": [
        "ocaml"
    ],
    "objdump": [
        "objdump"
    ],
    "mm": [
        "objective-c++",
        "xml"
    ],
    "sj": [
        "objective-j"
    ],
    "omgrofl": [
        "omgrofl"
    ],
    "opa": [
        "opa"
    ],
    "opal": [
        "opal"
    ],
    "opencl": [
        "opencl"
    ],
    "p": [
        "openedge abl"
    ],
    "scad": [
        "openscad"
    ],
    "org": [
        "org"
    ],
    "ox": [
        "ox"
    ],
    "oxh": [
        "ox"
    ],
    "oxo": [
        "ox"
    ],
    "oxygene": [
        "oxygene"
    ],
    "oz": [
        "oz"
    ],
    "pwn": [
        "pawn"
    ],
    "aw": [
        "php"
    ],
    "ctp": [
        "php"
    ],
    "php3": [
        "php"
    ],
    "php4": [
        "php"
    ],
    "php5": [
        "php"
    ],
    "phpt": [
        "php"
    ],
    "pls": [
        "plsql"
    ],
    "pkb": [
        "plsql"
    ],
    "pks": [
        "plsql"
    ],
    "plb": [
        "plsql"
    ],
    "plsql": [
        "plsql"
    ],
    "sql": [
        "plsql",
        "plpgsql",
        "sql",
        "sqlpl"
    ],
    "pan": [
        "pan"
    ],
    "psc": [
        "papyrus"
    ],
    "parrot": [
        "parrot"
    ],
    "pasm": [
        "parrot assembly"
    ],
    "pir": [
        "parrot internal representation"
    ],
    "pas": [
        "pascal"
    ],
    "dfm": [
        "pascal"
    ],
    "dpr": [
        "pascal"
    ],
    "lpr": [
        "pascal"
    ],
    "pp": [
        "pascal",
        "puppet"
    ],
    "pl": [
        "perl",
        "perl6",
        "prolog"
    ],
    "cgi": [
        "perl",
        "python",
        "shell"
    ],
    "perl": [
        "perl"
    ],
    "ph": [
        "perl"
    ],
    "plx": [
        "perl"
    ],
    "pm": [
        "perl",
        "perl6"
    ],
    "pod": [
        "perl",
        "pod"
    ],
    "psgi": [
        "perl"
    ],
    "t": [
        "perl",
        "perl6",
        "turing"
    ],
    "6pl": [
        "perl6"
    ],
    "6pm": [
        "perl6"
    ],
    "nqp": [
        "perl6"
    ],
    "p6": [
        "perl6"
    ],
    "p6l": [
        "perl6"
    ],
    "p6m": [
        "perl6"
    ],
    "pl6": [
        "perl6"
    ],
    "pm6": [
        "perl6"
    ],
    "pig": [
        "piglatin"
    ],
    "pike": [
        "pike"
    ],
    "pmod": [
        "pike"
    ],
    "pogo": [
        "pogoscript"
    ],
    "ps": [
        "postscript"
    ],
    "eps": [
        "postscript"
    ],
    "ps1": [
        "powershell"
    ],
    "psd1": [
        "powershell"
    ],
    "psm1": [
        "powershell"
    ],
    "pde": [
        "processing"
    ],
    "prolog": [
        "prolog"
    ],
    "spin": [
        "propeller spin"
    ],
    "proto": [
        "protocol buffer"
    ],
    "pub": [
        "public key"
    ],
    "pd": [
        "pure data"
    ],
    "pb": [
        "purebasic"
    ],
    "pbi": [
        "purebasic"
    ],
    "purs": [
        "purescript"
    ],
    "py": [
        "python"
    ],
    "gyp": [
        "python"
    ],
    "lmi": [
        "python"
    ],
    "pyde": [
        "python"
    ],
    "pyp": [
        "python"
    ],
    "pyt": [
        "python"
    ],
    "pyw": [
        "python"
    ],
    "tac": [
        "python"
    ],
    "wsgi": [
        "python"
    ],
    "xpy": [
        "python"
    ],
    "pytb": [
        "python traceback"
    ],
    "qml": [
        "qml"
    ],
    "pri": [
        "qmake"
    ],
    "r": [
        "r",
        "rebol"
    ],
    "rd": [
        "r"
    ],
    "rsx": [
        "r"
    ],
    "raml": [
        "raml"
    ],
    "rdoc": [
        "rdoc"
    ],
    "rbbas": [
        "realbasic"
    ],
    "rbfrm": [
        "realbasic"
    ],
    "rbmnu": [
        "realbasic"
    ],
    "rbres": [
        "realbasic"
    ],
    "rbtbar": [
        "realbasic"
    ],
    "rbuistate": [
        "realbasic"
    ],
    "rhtml": [
        "rhtml"
    ],
    "rmd": [
        "rmarkdown"
    ],
    "rkt": [
        "racket"
    ],
    "rktd": [
        "racket"
    ],
    "rktl": [
        "racket"
    ],
    "scrbl": [
        "racket"
    ],
    "rl": [
        "ragel in ruby host"
    ],
    "raw": [
        "raw token data"
    ],
    "reb": [
        "rebol"
    ],
    "r2": [
        "rebol"
    ],
    "r3": [
        "rebol"
    ],
    "rebol": [
        "rebol"
    ],
    "red": [
        "red"
    ],
    "reds": [
        "red"
    ],
    "cw": [
        "redcode"
    ],
    "rs": [
        "renderscript",
        "rust"
    ],
    "rsh": [
        "renderscript"
    ],
    "robot": [
        "robotframework"
    ],
    "rg": [
        "rouge"
    ],
    "rb": [
        "ruby"
    ],
    "builder": [
        "ruby"
    ],
    "gemspec": [
        "ruby"
    ],
    "god": [
        "ruby"
    ],
    "irbrc": [
        "ruby"
    ],
    "jbuilder": [
        "ruby"
    ],
    "mspec": [
        "ruby"
    ],
    "pluginspec": [
        "ruby",
        "xml"
    ],
    "podspec": [
        "ruby"
    ],
    "rabl": [
        "ruby"
    ],
    "rake": [
        "ruby"
    ],
    "rbuild": [
        "ruby"
    ],
    "rbw": [
        "ruby"
    ],
    "rbx": [
        "ruby"
    ],
    "ru": [
        "ruby"
    ],
    "ruby": [
        "ruby"
    ],
    "thor": [
        "ruby"
    ],
    "watchr": [
        "ruby"
    ],
    "sas": [
        "sas"
    ],
    "scss": [
        "scss"
    ],
    "sparql": [
        "sparql"
    ],
    "rq": [
        "sparql"
    ],
    "sqf": [
        "sqf"
    ],
    "hqf": [
        "sqf"
    ],
    "cql": [
        "sql"
    ],
    "ddl": [
        "sql"
    ],
    "prc": [
        "sql"
    ],
    "tab": [
        "sql"
    ],
    "udf": [
        "sql"
    ],
    "viw": [
        "sql"
    ],
    "db2": [
        "sqlpl"
    ],
    "ston": [
        "ston"
    ],
    "svg": [
        "svg"
    ],
    "sage": [
        "sage"
    ],
    "sagews": [
        "sage"
    ],
    "sls": [
        "saltstack",
        "scheme"
    ],
    "sass": [
        "sass"
    ],
    "scala": [
        "scala"
    ],
    "sbt": [
        "scala"
    ],
    "sc": [
        "scala",
        "supercollider"
    ],
    "scaml": [
        "scaml"
    ],
    "scm": [
        "scheme"
    ],
    "sld": [
        "scheme"
    ],
    "sps": [
        "scheme"
    ],
    "ss": [
        "scheme"
    ],
    "sci": [
        "scilab"
    ],
    "sce": [
        "scilab"
    ],
    "self": [
        "self"
    ],
    "sh": [
        "shell"
    ],
    "bash": [
        "shell"
    ],
    "bats": [
        "shell"
    ],
    "command": [
        "shell"
    ],
    "ksh": [
        "shell"
    ],
    "tmux": [
        "shell"
    ],
    "tool": [
        "shell"
    ],
    "zsh": [
        "shell"
    ],
    "sh-session": [
        "shellsession"
    ],
    "shen": [
        "shen"
    ],
    "sl": [
        "slash"
    ],
    "slim": [
        "slim"
    ],
    "tpl": [
        "smarty"
    ],
    "sp": [
        "sourcepawn"
    ],
    "sma": [
        "sourcepawn"
    ],
    "nut": [
        "squirrel"
    ],
    "ML": [
        "standard ml"
    ],
    "fun": [
        "standard ml"
    ],
    "sig": [
        "standard ml"
    ],
    "sml": [
        "standard ml"
    ],
    "do": [
        "stata"
    ],
    "ado": [
        "stata"
    ],
    "doh": [
        "stata"
    ],
    "ihlp": [
        "stata"
    ],
    "mata": [
        "stata"
    ],
    "matah": [
        "stata"
    ],
    "sthlp": [
        "stata"
    ],
    "styl": [
        "stylus"
    ],
    "scd": [
        "supercollider"
    ],
    "swift": [
        "swift"
    ],
    "sv": [
        "systemverilog"
    ],
    "svh": [
        "systemverilog"
    ],
    "vh": [
        "systemverilog"
    ],
    "toml": [
        "toml"
    ],
    "txl": [
        "txl"
    ],
    "tcl": [
        "tcl"
    ],
    "adp": [
        "tcl"
    ],
    "tm": [
        "tcl"
    ],
    "tcsh": [
        "tcsh"
    ],
    "csh": [
        "tcsh"
    ],
    "tex": [
        "tex"
    ],
    "aux": [
        "tex"
    ],
    "bbx": [
        "tex"
    ],
    "bib": [
        "tex"
    ],
    "cbx": [
        "tex"
    ],
    "dtx": [
        "tex"
    ],
    "ins": [
        "tex"
    ],
    "lbx": [
        "tex"
    ],
    "ltx": [
        "tex"
    ],
    "mkii": [
        "tex"
    ],
    "mkiv": [
        "tex"
    ],
    "mkvi": [
        "tex"
    ],
    "sty": [
        "tex"
    ],
    "toc": [
        "tex"
    ],
    "tea": [
        "tea"
    ],
    "txt": [
        "text"
    ],
    "textile": [
        "textile"
    ],
    "thrift": [
        "thrift"
    ],
    "tu": [
        "turing"
    ],
    "ttl": [
        "turtle"
    ],
    "twig": [
        "twig"
    ],
    "ts": [
        "typescript",
        "xml"
    ],
    "upc": [
        "unified parallel c"
    ],
    "uc": [
        "unrealscript"
    ],
    "vcl": [
        "vcl"
    ],
    "vhdl": [
        "vhdl"
    ],
    "vhd": [
        "vhdl"
    ],
    "vhf": [
        "vhdl"
    ],
    "vhi": [
        "vhdl"
    ],
    "vho": [
        "vhdl"
    ],
    "vhs": [
        "vhdl"
    ],
    "vht": [
        "vhdl"
    ],
    "vhw": [
        "vhdl"
    ],
    "vala": [
        "vala"
    ],
    "vapi": [
        "vala"
    ],
    "veo": [
        "verilog"
    ],
    "vim": [
        "viml"
    ],
    "vb": [
        "visual basic"
    ],
    "bas": [
        "visual basic"
    ],
    "frm": [
        "visual basic"
    ],
    "frx": [
        "visual basic"
    ],
    "vba": [
        "visual basic"
    ],
    "vbhtml": [
        "visual basic"
    ],
    "vbs": [
        "visual basic"
    ],
    "volt": [
        "volt"
    ],
    "owl": [
        "web ontology language"
    ],
    "webidl": [
        "webidl"
    ],
    "xc": [
        "xc"
    ],
    "xml": [
        "xml"
    ],
    "ant": [
        "xml"
    ],
    "axml": [
        "xml"
    ],
    "ccxml": [
        "xml"
    ],
    "clixml": [
        "xml"
    ],
    "cproject": [
        "xml"
    ],
    "csproj": [
        "xml"
    ],
    "ct": [
        "xml"
    ],
    "dita": [
        "xml"
    ],
    "ditamap": [
        "xml"
    ],
    "ditaval": [
        "xml"
    ],
    "dll.config": [
        "xml"
    ],
    "filters": [
        "xml"
    ],
    "fsproj": [
        "xml"
    ],
    "fxml": [
        "xml"
    ],
    "glade": [
        "xml"
    ],
    "grxml": [
        "xml"
    ],
    "ivy": [
        "xml"
    ],
    "jelly": [
        "xml"
    ],
    "kml": [
        "xml"
    ],
    "launch": [
        "xml"
    ],
    "mxml": [
        "xml"
    ],
    "nproj": [
        "xml"
    ],
    "nuspec": [
        "xml"
    ],
    "odd": [
        "xml"
    ],
    "osm": [
        "xml"
    ],
    "plist": [
        "xml"
    ],
    "ps1xml": [
        "xml"
    ],
    "psc1": [
        "xml"
    ],
    "pt": [
        "xml"
    ],
    "rdf": [
        "xml"
    ],
    "rss": [
        "xml"
    ],
    "scxml": [
        "xml"
    ],
    "srdf": [
        "xml"
    ],
    "storyboard": [
        "xml"
    ],
    "stTheme": [
        "xml"
    ],
    "sublime-snippet": [
        "xml"
    ],
    "targets": [
        "xml"
    ],
    "tmCommand": [
        "xml"
    ],
    "tml": [
        "xml"
    ],
    "tmLanguage": [
        "xml"
    ],
    "tmPreferences": [
        "xml"
    ],
    "tmSnippet": [
        "xml"
    ],
    "tmTheme": [
        "xml"
    ],
    "ui": [
        "xml"
    ],
    "urdf": [
        "xml"
    ],
    "vbproj": [
        "xml"
    ],
    "vcxproj": [
        "xml"
    ],
    "vxml": [
        "xml"
    ],
    "wsdl": [
        "xml"
    ],
    "wsf": [
        "xml"
    ],
    "wxi": [
        "xml"
    ],
    "wxl": [
        "xml"
    ],
    "wxs": [
        "xml"
    ],
    "x3d": [
        "xml"
    ],
    "xacro": [
        "xml"
    ],
    "xaml": [
        "xml"
    ],
    "xib": [
        "xml"
    ],
    "xlf": [
        "xml"
    ],
    "xliff": [
        "xml"
    ],
    "xmi": [
        "xml"
    ],
    "xml.dist": [
        "xml"
    ],
    "xsd": [
        "xml"
    ],
    "xul": [
        "xml"
    ],
    "zcml": [
        "xml"
    ],
    "xpl": [
        "xproc"
    ],
    "xproc": [
        "xproc"
    ],
    "xquery": [
        "xquery"
    ],
    "xq": [
        "xquery"
    ],
    "xql": [
        "xquery"
    ],
    "xqm": [
        "xquery"
    ],
    "xqy": [
        "xquery"
    ],
    "xs": [
        "xs"
    ],
    "xslt": [
        "xslt"
    ],
    "xsl": [
        "xslt"
    ],
    "xojo_code": [
        "xojo"
    ],
    "xojo_menu": [
        "xojo"
    ],
    "xojo_report": [
        "xojo"
    ],
    "xojo_script": [
        "xojo"
    ],
    "xojo_toolbar": [
        "xojo"
    ],
    "xojo_window": [
        "xojo"
    ],
    "xtend": [
        "xtend"
    ],
    "yml": [
        "yaml"
    ],
    "reek": [
        "yaml"
    ],
    "rviz": [
        "yaml"
    ],
    "yaml": [
        "yaml"
    ],
    "zep": [
        "zephir"
    ],
    "zimpl": [
        "zimpl"
    ],
    "zmpl": [
        "zimpl"
    ],
    "zpl": [
        "zimpl"
    ],
    "desktop": [
        "desktop"
    ],
    "desktop.in": [
        "desktop"
    ],
    "ec": [
        "ec"
    ],
    "eh": [
        "ec"
    ],
    "edn": [
        "edn"
    ],
    "fish": [
        "fish"
    ],
    "mu": [
        "mupad"
    ],
    "nc": [
        "nesc"
    ],
    "ooc": [
        "ooc"
    ],
    "rst": [
        "restructuredtext"
    ],
    "rest": [
        "restructuredtext"
    ],
    "wisp": [
        "wisp"
    ],
    "prg": [
        "xbase"
    ]
}



import json
import os
import re

import mistune

from ..models.book import Book
from ..renderer.renderer_summary import SummaryRenderer
from ..utils.error import error, file_not_found_error

count_sum = 0
data_level = []
iter_count = 0


def parse_summary(book: Book):
    """解析目录"""
    # 得到目录 html
    global count_sum, iter_count, data_level
    with open(book.summary_path, encoding="utf-8") as f:
        summary_dict = []
        page = f.read()
        page = re.sub(r"<!--.*?-->", "", page, flags=re.S)
        page = re.sub(r"^(?!\s*\*|#).*", "", page, flags=re.M)
        page = re.sub(r"\s*[\n|\r\n]+", r"\n\n", page)
        summary = SummaryRenderer()
        md = mistune.Markdown(renderer=summary)
        md.parse(page)
        count = 0
        for c1 in summary.toc_tree:
            if count == 0:
                c1[0] = c1[0] if c1[0] else book.i18n.get("SUMMARY")
            count += 1
            count_sum += 1
            iter_count = 0
            data_level = data_level[:iter_count]
            data_level.append(str(count))

            header_text, sub = c1

            summary_dict.append({"title": header_text, "articles": [], "data_level": f"{'.'.join(data_level)}"})

            if len(sub) == 0:
                continue
            try:
                data_json = json.loads(sub.replace("][", "],["), encoding="utf-8")
                summary_dict[-1]["articles"] = _iter_list(book, data_json)
            except Exception as e:
                error(f"解析目录异常，请检查目录结构：\n{sub}\n{e}")
    book.summary = summary_dict


def _iter_list(book, data_json):
    """迭代列表项

    :return:
    """
    global count_sum, iter_count, data_level
    sub = []
    count = 0
    iter_count += 1
    for item in data_json:
        if type(item[0]) is str:
            count_sum += 1
            count += 1
            data_level = data_level[:iter_count]
            data_level.append(str(count))
            data_level_str = '.'.join(data_level)
            book.summary_level_list = [data_level_str, item[0], item[2]]
            sub.append({"title": item[2], "ref": item[0], "articles": [], "data_level": f"{data_level_str}"})
            pass
        else:
            sub[-1]["articles"] = _iter_list(book, item)
            pass
    iter_count -= 1
    return sub


def is_summary_exist(book: Book):
    """验证目录文件是否存在

    :param book:
    :return:
    """
    book_summary = os.path.join(book.book_path, "SUMMARY.md")
    if not os.path.isfile(book_summary):
        file_not_found_error(book_summary, "必须存在目录，请检查文件名称是否准确：SUMMARY.md")

    book.summary_path = book_summary



import logging
import os
import shutil


def rmdir(file_path: str):
    """删除目录

    :param file_path:
    :return:
    """
    if os.path.isdir(file_path):
        logging.debug(f"清理目录：{file_path}")
        shutil.rmtree(file_path)


def mkdir(file_path: str):
    """创建目录

    :param file_path:
    :return:
    """
    logging.debug(f"创建目录：{file_path}")
    os.makedirs(file_path)


def copytree(src, dst, *ignore):
    """复制目录"""
    shutil.copytree(src, dst, ignore=shutil.ignore_patterns(*ignore))


def copy(src, dst):
    """复制文件到文件夹"""
    if not os.path.isdir(dst):
        mkdir(dst)
    shutil.copy(src, dst)


def is_file_exist(root: str, filename):
    """验证文件是否存在

    :param root:
    :param filename:
    :return:
    """
    path = os.path.join(root, filename)
    if os.path.isfile(path):
        return True
    else:
        return False
